/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * cvmmulticlassmodel.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * cvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _CVMMULTICLASSMODEL_H_
#define _CVMMULTICLASSMODEL_H_
#include "multiclassmodel_interface.h"
#include <boost/serialization/serialization.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/serialization/shared_ptr.hpp> 

using namespace arma;

// just a simple container object (lazy version)
class cvm_multiclass_model: public IMultiClassModelInterface
{
	public:
		mat  CalculateModelPredictions(const IKernelMatrixInterface &oKernelInterface) const;
		void GenerateMultiClassModels(ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label);
		const std::vector<std::vector<double> >& GetReferenceData(void) const {return sm_vvReferenceData; };
		arma::vec GetUniqueLabels(void)  {return sm_vUniqueLabel;};
		std::vector<cvm_model> GetModels(void)  {return sm_Model; } ;
		arma::uvec GetReferenceIndices(void) {return sm_vReferenceIndices;};

		void SetModels(const std::vector<cvm_model>& vModels) { sm_Model = vModels;}
		void SetReferenceData(const std::vector<std::vector<double> >& vData) { sm_vvReferenceData = vData;}
		void SetUniqueLabels(const arma::vec& vLabels) { sm_vUniqueLabel = vLabels;}
		void SetReferenceIndices(const arma::uvec&  vIndices) {sm_vReferenceIndices = vIndices;}

		std::vector<cvm_model> sm_Model;
		std::vector<std::vector<double> > sm_vvReferenceData;
		arma::uvec sm_vReferenceIndices;		
		arma::vec sm_vUniqueLabel;		
	protected:

	private:
		// Allow serialization to access non-public data members.
		friend class boost::serialization::access;
		cvm_model CalculateClassificationModel (int i,ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const;
		template< class Archive >
		inline void save( Archive& ar, const unsigned int ) const;
		template< class Archive >
		inline void load( Archive& ar, unsigned int const );

		BOOST_SERIALIZATION_SPLIT_MEMBER()
};


		template< class Archive >
		inline void cvm_multiclass_model::save( Archive& ar, const unsigned int ) const
{
			std::cout << "Save cvm_multiclass_model" << std::endl;
			boost::serialization::void_cast_register<cvm_multiclass_model, IMultiClassModelInterface>();
			const cvm_multiclass_model& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			typedef std::vector<double> stdvec;
			typedef std::vector<unsigned int> uintvec;
			stdvec vDoubleVec;		
			uintvec vVec2;
			vDoubleVec = conv_to< stdvec  >::from(oClass.sm_vUniqueLabel);
			vVec2      = conv_to< uintvec >::from(oClass.sm_vReferenceIndices);
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-s] "));
			ar & vDoubleVec;  // Simply serialize the data members of Obj
			ar & vVec2;
			ar & oClass.sm_Model;
			ar & oClass.sm_vvReferenceData;
}

		template< class Archive >
		inline void cvm_multiclass_model::load( Archive& ar, unsigned int const )
{
			std::cout << "OAR"<< std::endl;
			std::cout << "Load cvm_multiclass_model" << std::endl;
			boost::serialization::void_cast_register<cvm_multiclass_model, IMultiClassModelInterface>();
			cvm_multiclass_model& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			std::vector<double> vDoubleVec;
			std::vector<unsigned int> vVec2;
			ar & vDoubleVec;
			ar & vVec2;
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-l] "));
			oClass.sm_vUniqueLabel = zeros<vec>(vDoubleVec.size());
			oClass.sm_vUniqueLabel = conv_to< arma::vec >::from(vDoubleVec);
			oClass.sm_vReferenceIndices = conv_to< arma::uvec >::from(vVec2);
			ar & oClass.sm_Model;
			ar & oClass.sm_vvReferenceData;
}
BOOST_CLASS_VERSION(cvm_multiclass_model, 1)
#endif // _CVMMULTICLASSMODEL_H_
