/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * cvmmulticlassmodel_each.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * cvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _CVMMULTICLASSMODEL_EACH_H_
#define _CVMMULTICLASSMODEL_EACH_H_
#include "multiclassmodel_interface.h"
#include <boost/serialization/serialization.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/serialization/shared_ptr.hpp> 

using namespace arma;


// just a simple container object (lazy version)
class cvm_multiclass_model_each: public IMultiClassModelInterface
{
	public:
		mat  CalculateModelPredictions(const IKernelMatrixInterface &oKernelInterface) const;
		void GenerateMultiClassModels(ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label);

		// getter
		const std::vector<std::vector<double> >& GetReferenceData(void) const {return sm_vvReferenceData; } ;
		arma::vec GetUniqueLabels(void)  {return sm_vUniqueLabel;} ;
		std::vector<cvm_model> GetModels(void)  {return sm_Model; } ;
		arma::uvec GetReferenceIndices(void) {return sm_vReferenceIndices;};
		std::vector<std::vector<unsigned int> > GetClassIndices(void) const {return m_vvClassIndices;};

		// setter
		void SetModels(const std::vector<cvm_model>& vModels) { sm_Model = vModels;}
		void SetReferenceData(const std::vector<std::vector<double> >& vData) { sm_vvReferenceData = vData;}
		void SetUniqueLabels(const arma::vec& vLabels) { sm_vUniqueLabel = vLabels;}
		void SetReferenceIndices(const arma::uvec&  vIndices) {sm_vReferenceIndices = vIndices;}
		void SetClassIndices(const std::vector<std::vector<unsigned int> >& vClassIndices) {m_vvClassIndices=vClassIndices;};

		std::vector<cvm_model> sm_Model;
		std::vector<std::vector<double> > sm_vvReferenceData;
		std::vector<std::vector<unsigned int> > m_vvClassIndices;
		arma::uvec sm_vReferenceIndices;		
		arma::vec sm_vUniqueLabel;		

		template< typename t_Archive >
		void save( t_Archive& archive, const unsigned int) const;

		template< typename t_Archive >
		void load( t_Archive& archive, unsigned int const );

		BOOST_SERIALIZATION_SPLIT_MEMBER()
	protected:

	private:
		// Allow serialization to access non-public data members.
		friend class boost::serialization::access;

		cvm_model CalculateClassificationModel (int i,ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const;
};
BOOST_CLASS_VERSION(cvm_multiclass_model_each, 1)

template<typename Archive>
		void cvm_multiclass_model_each::save(Archive& ar, const unsigned version) const {
			const cvm_multiclass_model_each& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			typedef std::vector<double> stdvec;
			typedef std::vector<unsigned int> uintvec;
			stdvec vDoubleVec;		
			uintvec vVec2;
			vDoubleVec = conv_to< stdvec  >::from(oClass.sm_vUniqueLabel);
			vVec2      = conv_to< uintvec >::from(oClass.sm_vReferenceIndices);
			//std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-s] "));
			ar & vDoubleVec;  // Simply serialize the data members of Obj
			ar & vVec2;
			ar & oClass.sm_Model;
			ar & oClass.sm_vvReferenceData;
			ar & oClass.m_vvClassIndices;
		}

template<typename Archive>
			void cvm_multiclass_model_each::load(Archive & ar, const unsigned int version) { 
			std::cout << "EAE"<< std::endl;
			cvm_multiclass_model_each& oClass = *this;
			ar & boost::serialization::base_object<IMultiClassModelInterface>(*this);
			std::vector<double> vDoubleVec;
			std::vector<unsigned int> vVec2;
			std::cout << " - 1 - "<< std::endl;
			ar & vDoubleVec;
			std::cout << " - 2 - "<< std::endl;
			ar & vVec2;
			std::cout << " - 3 - "<< std::endl;
			std::cout << " - 3a - "<< vDoubleVec.size() <<std::endl;
			std::copy(vDoubleVec.begin(),vDoubleVec.end(),std::ostream_iterator<double>(std::cout," [multi-l] "));
			oClass.sm_vUniqueLabel = zeros<vec>(vDoubleVec.size());
			//oClass.sm_vUniqueLabel = conv_to< arma::vec >::from(vDoubleVec);
			for(int i=0; i<vDoubleVec.size();i++)
			{
				oClass.sm_vUniqueLabel[i] = vDoubleVec[i];
			}
			std::cout << " - 4 - "<< std::endl;
			oClass.sm_vReferenceIndices = zeros<uvec>(vVec2.size());
//			oClass.sm_vReferenceIndices = conv_to< arma::uvec >::from(vVec2);
			for(int i=0; i<vVec2.size();i++)
			{
				oClass.sm_vReferenceIndices[i] = vVec2[i];
			}

			std::cout << " - 5 - "<< std::endl;
			ar & oClass.sm_Model;
			std::cout << " - 6 - "<< std::endl;
			ar & oClass.sm_vvReferenceData;
			std::cout << " - 7 - "<< std::endl;
			ar & oClass.m_vvClassIndices;
			std::cout << " - 8 - "<< std::endl;
		}
	
#endif // _CVMMULTICLASSMODEL_EACH_H_
