/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * EIG_NY.hpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Implementation of an elm kernel in acc to
 * Frenay & Verleysen, Parameter Free kernel in extreme learnin ...,
 * Neurocomputing
 * Author: Frank-Michael Schleif
 */
#ifndef _EIG_NY_HPP_
#define _EIG_NY_HPP_

#include <iostream>
#include <armadillo>
#include <cmath>

using namespace std;
using namespace arma;

/*
% computes the exact eigenvalue decomposition of a Nystroem approximated
% matrix
% Input agruments:
%   ny_K is a Nystroem approximated matrix of form
%     ny_K{1}=K_Nm
%     ny_K{2}=K_mm^-1
% Output agruments:
%   C is the matrix of eigenvectors
%   A is the diagonal matrix of eigenvalues
*/

void eig_ny(const mat &C, const mat &Wpinv, mat &E, mat &V);
#include "eig_ny_impl.hpp"
#endif
