/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * ELM_KERNEL_IMPL.hpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Implementation of an elm kernel in acc to
 * Frenay & Verleysen, Parameter Free kernel in extreme learnin ...,
 * Neurocomputing
 * Author: Frank-Michael Schleif
 */
#ifndef _ELM_KERNEL_IMPL_HPP_
#define _ELM_KERNEL_IMPL_HPP_
#include <armadillo>
#include "elm_kernel.hpp"

inline mat CalculateELMKernel(mat coord)
{
 	 unsigned int n = coord.n_rows;
 	 double      sig= 1e10; // kernel parameter should be large
	 mat K         = coord*coord.t();
	 colvec vD     = K.diag();
	 mat KdiagX    = repmat(vD,1,n);
	 double dSig   = 1/(2*sig);
	 mat aELM      = 2/M_PI * asin((K+1)/sqrt((dSig + 1 + KdiagX) % (dSig + 1 + KdiagX.t())) );
	 vD            = aELM.diag();
	 KdiagX        = repmat(vD,1,n);
	 K             = aELM / sqrt(KdiagX % KdiagX.t()) ;
	 return K;
}
#endif
