/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * ELM_KERNEL_XY_IMPL.hpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Implementation of an elm kernel in acc to
 * Frenay & Verleysen, Parameter Free kernel in extreme learnin ...,
 * Neurocomputing
 * Author: Frank-Michael Schleif
 */
#ifndef _ELM_KERNEL_XY_IMPL_HPP_
#define _ELM_KERNEL_XY_IMPL_HPP_
#include <armadillo>
#include "math_stat_helper.hpp"
#include "elm_kernel_xy.hpp"
inline mat CalculateELMKernelXY(mat X, mat Y)
{
    double dSig = 1.0/(2.0*1e10);
	vec KxDiag = sum(X % X,1);
    vec KyDiag = sum(Y % Y,1);
    //mat K = 2.0/M_PI*asin( clamp( (1.0+X*Y.t()) / sqrt((dSig+1.0+KxDiag)* (dSig+1.0+KyDiag).t() ),-1.0,1.0));
	// -->  a bit more memory efficient
	mat K  = 1.0/(sqrt((dSig+1.0+KxDiag)* (dSig+1.0+KyDiag).t() ));
	K %= (1.0+X*Y.t());
	K = 2.0/M_PI*asin( clamp( K,-1.0,1.0));
    return K;
}
#endif
