/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * kernel-matrix.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _KERNEL_MATRIX_H_
#define _KERNEL_MATRIX_H_
#include <armadillo>
#include "kernel-matrix_interface.h"

using namespace arma;
class KernelMatrix: public IKernelMatrixInterface
{
	public:
		KernelMatrix(int iKernelType);
		~KernelMatrix();
		// generate the kernel matrix from the given vectorial data
		// if needed optional parameters can be provided in Parameter   
		bool Initialize(boost::shared_ptr<mat> mData, const boost::any &Parameter);
		// const mat&  GetKernelMatrix(void) const;
		mat  Evaluate(const uvec &X, const uvec &Y) const;
		int  GetKernelType(void) const;
		void size(unsigned int &rows, unsigned int &cols) const;
		// the kernel (matrix) has to be initialized first
		bool GetStatus(void) const;
		IKernelMatrixInterface* clone() const {
    		return new KernelMatrix(*this);
		}	
		//mat GetParameter (int iParameter) const {return zeros<mat>(1,1);}
protected:

private:
	//bool GenerateKernelMatrix(const mat &DataX, const mat &DataY, const mat &Parameter);
	bool m_status;
	int m_iKernelType;
//	mat m_KernelMatrix;
	boost::shared_ptr<mat> m_DataMatrix;
};

#endif // _KERNEL_MATRIX_H_

