/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * kernel-matrix-interface.h
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _KERNEL_MATRIX_INTERFACE_H_
#define _KERNEL_MATRIX_INTERFACE_H_
#include <armadillo>
#include <boost/any.hpp>
#include <boost/shared_ptr.hpp>

using boost::any_cast;
using namespace arma;

class IKernelMatrixInterface
{
	public:
		virtual ~IKernelMatrixInterface() {};
		// generate the kernel matrix from the given vectorial data
		// if needed optional parameters can be provided in Parameter
		// virtual bool Initialize(const mat &DataX, const mat &DataY, const CParam &Parameter)=0;
/*		virtual bool Initialize(const mat &DataX, const mat &DataY) = 0;
		virtual bool Initialize(const mat &DataX, const mat &DataY, const boost::any &Parameter) {};
*/
//		virtual const mat& GetKernelMatrix(void) const {return mDummy;};
		virtual bool Initialize(boost::shared_ptr<mat> mData, const boost::any &Parameter)=0;
		virtual int  GetKernelType(void) const = 0;
		virtual mat  Evaluate(const uvec &X, const uvec &Y) const = 0;
		virtual void size(unsigned int &rows, unsigned int &cols) const = 0;
		// the kernel (matrix) has to be initialized first
		virtual bool GetStatus(void) const = 0;
		virtual IKernelMatrixInterface* clone() const = 0;
//		virtual mat GetParameter (int iParameter) const =0;
	private:
		mat mDummy;
};

#endif // _KERNEL_MATRIX_INTERFACE_H_

