/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * math_stat_helper_IMPL.hpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MATH_STAT_HELPER_IMPL_HPP_
#define _MATH_STAT_HELPER_IMPL_HPP_
#ifndef M_PI
#define 	M_PI   3.14159265358979323846	/* pi */
#endif
#include <armadillo>
#include "math_stat_helper.hpp"
/*	mat Test(2,2);
	Test << 0.165300 << 0.454037 << endr
		 << 0.688782 << 0.036549 << endr;
	mat TestCDF = normalCDF(Test);
	std::cout << TestCDF;
	return;
	//    0.5656    0.6751
    //    0.7545    0.5146
*/

// normal cummulative distribution function special case for N(0,1) only
inline mat normalCDF(mat Matrix)
{
   //return 0.5 * erfc(-value * M_SQRT1_2);

	// constants
    double a1 =  0.254829592;
    double a2 = -0.284496736;
    double a3 =  1.421413741;
    double a4 = -1.453152027;
    double a5 =  1.061405429;
    double p  =  0.3275911;
 
    // Save the elements
    mat SignMat = sign(Matrix);
    Matrix      = abs(Matrix)/sqrt(2.0);
 
    // A&S formula 7.1.26; Handbook of Mathematical Functions by Abramowitz and Stegun
    mat t = 1.0/(1.0 + p*Matrix);
    mat y = 1.0 - (((((a5*t + a4) % t) + a3) % t + a2) % t + a1) % t % exp(-Matrix % Matrix);
 
    return 0.5*(1.0 + SignMat % y);
}

/*	mat Test(2,2);
	Test << 0.165300 << 0.454037 << endr
		 << 0.688782 << 0.036549 << endr;
	mat TestCDF = normalPDF(Test);
	std::cout << TestCDF;
	return;
	//  0.3935    0.3599
    // 0.3147    0.3987

*/
// normal probability density function special case for N(0,1) only
inline mat normalPDF(mat Matrix)
{
    double mu = 0;
    double sigma = 1;
	mat NormalizedMat = (Matrix - mu)/sigma;
    return exp(-0.5 * (NormalizedMat % NormalizedMat)) / (sqrt(2*M_PI) * sigma);
}
#endif
