/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * cvmmodel.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cvmmodel.h"
cvm_model::cvm_model(arma::vec Weights, double dBias, int iKernelType)
{
	m_SupportIndices = find(Weights);	
	m_Weights = Weights(m_SupportIndices);
	m_dBias   = dBias;
	m_iKernelType = iKernelType;		
}

vec cvm_model::CalculatePrediction(const IKernelMatrixInterface &oKernelInstance,arma::uvec ReferenceIndices) const
{
	arma::uvec vModelSupportIndices = GetSupportIndices();
	arma::uvec vSelector = zeros<uvec>(vModelSupportIndices.n_elem);
//	std::cout << vModelSupportIndices << std::endl;
	for(unsigned int i=0; i<vModelSupportIndices.n_elem;++i)
	{
		vSelector(i) = as_scalar(find(ReferenceIndices==vModelSupportIndices(i)));
	}
//	std::cout << ReferenceIndices << std::endl;
//	std::cout << vModelSupportIndices << std::endl;
	unsigned int rows=0, cols = 0;
	oKernelInstance.size(rows, cols);
	uvec vDummy;
	if(m_iKernelType ==2 || m_iKernelType ==3)
		vDummy = linspace<uvec>(0, cols-1, cols); // first elements in the full kernel are the training data
	else
		vDummy = linspace<uvec>(ReferenceIndices.n_elem, rows-1, rows-ReferenceIndices.n_elem); // first elements in the full kernel are the training data
	int Ns = vSelector.n_elem;
	if(Ns < 1000) // not to many support indices 
	{
		arma::mat KernelMapping = oKernelInstance.Evaluate(vSelector,vDummy);
		//	std::cout << KernelMapping <<std::endl;
		//std::cout << Predict(KernelMapping).t() << std::endl;
		return Predict(KernelMapping);  
	}
	else
	{
		int iNrPredictions= vDummy.n_elem;
		arma::vec vResult = zeros<vec>(iNrPredictions);		
		int iStart = 0;
		int iBlocksize = 10000;
		while(iStart < iNrPredictions)
		{
			int iEnd   = std::min(iStart+iBlocksize,iNrPredictions-1);
			uvec vBlock = linspace<uvec>(iStart, iEnd, iBlocksize);
			arma::mat KernelMapping = oKernelInstance.Evaluate(vSelector,vDummy.rows(vBlock));
			vResult.rows(vBlock) = Predict(KernelMapping);
			std::cout << " Prediction completed: " << 100.0*(double)iStart/(double)iNrPredictions << "\%\n";
			iStart = iEnd+1;
		}
		std::cout << " Prediction completed: " << "100.0\%\n";
		return vResult;
	}
}

vec cvm_model::CalculatePrediction(const mat &RawKernelMatrix) const
{
	return Predict(RawKernelMatrix);
}


arma::vec cvm_model::Predict(arma::mat Data) const
{
	//std::cout << Data.t() * m_Weights + m_dBias << std::endl;
	return Data.t() * m_Weights + m_dBias;
}

arma::vec cvm_model::GetWeights(void) const
{
	return m_Weights;
}

double cvm_model::GetBias(void) const
{
	return m_dBias;
}

int cvm_model::GetKernelType(void) const
{
	return m_iKernelType;
}

arma::uvec cvm_model::GetSupportIndices(void) const
{
	return m_SupportIndices;
}

