/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * cvmmulticlassmodel_each.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * cvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * cvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cvmmulticlassmodel_each.h"
#include "math_stat_helper.hpp"
#include "kernel-matrix_interface.h"
#include <vector>
#include <boost/bind.hpp>
#include <typeinfo>
using namespace std;

mat cvm_multiclass_model_each::CalculateModelPredictions(const IKernelMatrixInterface &oKernelInstance) const
{
  arma::vec vUniqueLabelFullModel     = sm_vUniqueLabel;
  std::vector<cvm_model> vCVMModels = sm_Model;
  std::vector<vec> vPredictions;
  arma::uvec ReferenceIndices         = sm_vReferenceIndices;
  unsigned int iRows = 0;
  unsigned int iCols = 0;
  int iKernel = oKernelInstance.GetKernelType();
  oKernelInstance.size(iRows,iCols);
  unsigned int N = iRows - ReferenceIndices.n_elem;
  vPredictions.resize(vUniqueLabelFullModel.size());

  unsigned int C = sm_vUniqueLabel.n_elem;
  int iCounter = 0;

  if(iKernel == 2)
  {
	  N = iCols;
  }
  mat mPredictionMatrix;
  if(iKernel != 2)
	mPredictionMatrix   = zeros<mat>(N ,vCVMModels.size());
  else
	mPredictionMatrix   = zeros<mat>(iCols ,vCVMModels.size());

  mat mVoteMatrix       = zeros<mat>(N,vCVMModels.size());
  mat mVotingOptions    = zeros<mat>(vCVMModels.size(),2);
  mat mPredictions      = zeros<mat>(N,vUniqueLabelFullModel.size());

  for(unsigned int k=0; k<C;k++)
  {
		for(unsigned int l=k+1; l<C; l++)
		{			
			bool bResult;
			const cvm_model &oCurrentModel = vCVMModels[iCounter];
			vec vCurrentPredictions;

			uvec vCurrentLabelIndices = conv_to< arma::uvec >::from(m_vvClassIndices[iCounter]);
			
			arma::uvec vModelSupportIndices = oCurrentModel.GetSupportIndices();
//			std::cout << vModelSupportIndices << std::endl;
			arma::uvec vSelector = zeros<uvec>(vModelSupportIndices.n_elem);
			for(unsigned int i=0; i<vModelSupportIndices.n_elem;++i)
			{
				vSelector(i) = as_scalar(find(ReferenceIndices==vCurrentLabelIndices(vModelSupportIndices(i))));
			}
/*			std::cout << "Selector: " << std::endl;
			std::cout << vSelector << std::endl;
	*/	
			uvec vDummy;
			if(iKernel ==2 || iKernel == 3)
				vDummy = linspace<uvec>(0, iCols-1, iCols); // first elements in the full kernel are the training data
			else
				vDummy = linspace<uvec>(ReferenceIndices.n_elem, iRows-1, iRows-ReferenceIndices.n_elem);
			
			const mat RawKernelMatrix = oKernelInstance.Evaluate(vSelector,vDummy);
			/*std::cout << "KernelMatrix:" << std::endl;			
			std::cout << RawKernelMatrix << std::endl;*/
			//std::cout << RawKernelMatrix.n_rows << " - " << RawKernelMatrix.n_cols <<std::endl;
			vCurrentPredictions = oCurrentModel.CalculatePrediction(RawKernelMatrix);

		
			mPredictionMatrix.col(iCounter) = vCurrentPredictions;
			vec vCurrentLabels;
			vCurrentLabels << vUniqueLabelFullModel(k) << vUniqueLabelFullModel(l);
			mVotingOptions.row(iCounter)    = vCurrentLabels.t();
			iCounter++;
		}				
	}
	umat mIndicator = 1-(mPredictionMatrix<0);
/*	std::cout << " *********** " << std::endl;
	std::cout << mPredictionMatrix << std::endl;
	std::cout << mVotingOptions << std::endl;	
	std::cout << mIndicator << std::endl;
	std::cout << " *********** " << std::endl;*/	
//			std::cout << "start voting ..."<<std::endl;
	
	for(unsigned int k=0;k<N;k++)
	{
		for(unsigned int l=0; l<iCounter;l++)
		{		
			mVoteMatrix(k,l) = mVotingOptions(l,mIndicator(k,l));
		}
	
    
		for(unsigned int l=0; l<C; l++)
		{
			uvec vTemp        = find(mVoteMatrix.row(k)==vUniqueLabelFullModel(l));
			mPredictions(k,l) = vTemp.n_elem;
		}	
	}
		
	//std::cout <<	mPredictions  << std::endl;
	
	uvec vCrispPrediction = zeros<uvec>(N);

	for(unsigned int k=0;k<N;k++)
	{
		uword  idx;
		vec vTemp = mPredictions.row(k).t();
		double dMaxValue = vTemp.max(idx);
		//std::cout <<  vUniqueLabelFullModel(idx) << std::endl;
		vCrispPrediction(k) = idx;
	}		    

//	std::cout << " - " << std::endl;
  mat PredictionResults = zeros<mat>(N,vUniqueLabelFullModel.size()+1);
  PredictionResults.cols(0,vUniqueLabelFullModel.size()-1) = mPredictions;
  PredictionResults.col(vUniqueLabelFullModel.size())      = conv_to<vec>::from(vCrispPrediction);
  return PredictionResults;
}

cvm_model cvm_multiclass_model_each::CalculateClassificationModel (int i,ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const vec &Label) const  
{  
	vec vNewLabelVector  = ones<vec>(Label.size(),1);
	uvec vValid = find(Label == i);
	vNewLabelVector(vValid) = -vNewLabelVector(vValid);
	oCVMTrain.Apply(oKernel,vNewLabelVector);
	std::cout << "*";
	std::cout << std::flush;
	return oCVMTrain.GetModel();
}


void cvm_multiclass_model_each::GenerateMultiClassModels(ICVMTrainInterface &oCVMTrain,const IKernelMatrixInterface &oKernel, const mat &Data, const vec &Label)
{
	cvm_multiclass_model_each &sFullModel = *this;
	arma::vec vUniqueLabel = unique(Label);
	std::vector<cvm_model> vCVMModels;
	std::vector<uvec>  vvClassIndices;
	std::cout << " - " << std::endl;

	int iKernel = oKernel.GetKernelType();
	IKernelMatrixInterface *oTwoClassKernel = NULL;
	oTwoClassKernel	 = oKernel.clone();

	unsigned int C = vUniqueLabel.n_elem;
	for(unsigned int k=0; k<C;k++)
	{
		for(unsigned int l=k+1; l<C; l++)
		{
			uvec vCurrentLabelIndices = find(Label == vUniqueLabel(k) || Label == vUniqueLabel(l));
			vec vCurrentLabeling      = Label(vCurrentLabelIndices);			
			arma::vec vCurrentUniqueLabel = unique(vCurrentLabeling);
			bool bResult;
			mat mSubDataSet           = Data.rows(vCurrentLabelIndices);
			if(iKernel == 2)
				mSubDataSet           = mSubDataSet.cols(vCurrentLabelIndices);
			boost::shared_ptr<mat> mSharedData = boost::shared_ptr<mat>(new mat(mSubDataSet));
			bResult = oTwoClassKernel->Initialize(mSharedData,zeros<mat>(1,1));//,(Data.rows(vCurrentLabelIndices)));
			//std::cout << oTwoClassKernel->Evaluate(vCurrentLabelIndices,vCurrentLabelIndices) << std::endl;
			cvm_model oCurrentCVMModel = CalculateClassificationModel(vUniqueLabel(k),oCVMTrain,*oTwoClassKernel,vCurrentLabeling);
			vCVMModels.push_back(oCurrentCVMModel);
			vvClassIndices.push_back(vCurrentLabelIndices);
		}
	}
	delete oTwoClassKernel;
	std::cout << " - " << std::endl;
	sFullModel.sm_vUniqueLabel = vUniqueLabel;
	sFullModel.sm_Model        = vCVMModels;

	// identify the reference data points we have to keep by
	// analyzing the support indices of the various models
	arma::uvec vUniqueSupportIndices; 
	for(unsigned int k=0; k<vCVMModels.size(); ++k)
	{
		if(k==0)
			vUniqueSupportIndices =	vvClassIndices[k](vCVMModels[k].GetSupportIndices ());
		else
			vUniqueSupportIndices = unique(join_vert(vUniqueSupportIndices,vvClassIndices[k](vCVMModels[k].GetSupportIndices ())));
	}
	sFullModel.sm_vReferenceIndices = vUniqueSupportIndices; 

	m_vvClassIndices.clear();
	typedef std::vector<unsigned int> uintvec;
	for(unsigned int k=0; k<vCVMModels.size(); ++k)
	{
		m_vvClassIndices.push_back(conv_to< uintvec >::from(vvClassIndices[k]));
	}
			
	// reference data are only needed for vectorial data
	// so we keep sFullModel.sm_vvReferenceData
	if(iKernel != 2)
	{						
		int NPoints = vUniqueSupportIndices.n_elem;
		typedef std::vector<double> stdvec;
		std::vector< stdvec > vvData;
		vvData.resize(NPoints);
		for(int k=0; k<NPoints; k++)
		{
			arma::vec vTemp    = vectorise(Data.row(vUniqueSupportIndices(k)));
			vvData[k] = arma::conv_to< stdvec >::from(vTemp);
		}
		sFullModel.sm_vvReferenceData = vvData;
	}
	else
	{
		sFullModel.sm_vvReferenceData.clear();
	}
}

