/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * kernel-matrix.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * pcvm_cpp is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "kernel-matrix.h"
#include "elm_kernel_xy.hpp"

KernelMatrix::KernelMatrix(int iKernelType)
{
	m_iKernelType = iKernelType;
	m_status      = false;
}

KernelMatrix::~KernelMatrix()
{
}


bool KernelMatrix::Initialize(boost::shared_ptr<mat> mData, const boost::any &AnyPar) 
{
/*	try
    {
        const mat &Parameters = (any_cast<const mat &>(AnyPar));
		m_status = GenerateKernelMatrix(DataX, DataY, Parameters);
    }
    catch(const boost::bad_any_cast &)
    {
        return false;
    }*/
	m_DataMatrix = mData;
//	std::cout << *m_DataMatrix << std::endl;
	return true;
}

/*const mat& KernelMatrix::GetKernelMatrix(void) const
{
	return m_KernelMatrix;
}*/

int KernelMatrix::GetKernelType(void) const
{
	return m_iKernelType;
}

bool KernelMatrix::GetStatus(void) const
{
	return m_status;
}

void KernelMatrix::size(unsigned int &rows, unsigned int &cols) const
{
	rows = m_DataMatrix->n_rows;
	cols = m_DataMatrix->n_cols;
}

mat  KernelMatrix::Evaluate(const uvec &X, const uvec &Y) const
{
	switch(m_iKernelType)
	{
		case 0:
			return m_DataMatrix->rows(X)*m_DataMatrix->rows(Y).t();
		case 1:
			return CalculateELMKernelXY(m_DataMatrix->rows(X),m_DataMatrix->rows(Y));
		case 2:
/*			std::cout << X << std::endl;
			std::cout << Y+1 << std::endl;*/
			return m_DataMatrix->submat(X,Y);
		default:
			return zeros<mat>(1,1);
			break;
			// nothing to do data is the kernel matrix already

	}
}
