/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * nystroem-kernel-matrix.cc
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * nystroem-kernel-matrix is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pcvm_cpp is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "nystroem-kernel-matrix.h"

NystroemKernelMatrix::NystroemKernelMatrix():KernelMatrix(3)
{
}

NystroemKernelMatrix::~NystroemKernelMatrix()
{

}

bool NystroemKernelMatrix::Initialize(boost::shared_ptr<mat> mData, const boost::any &Parameter) 
{
	bool bStatus = KernelMatrix::Initialize(mData, Parameter);
	m_Landmarks  = boost::any_cast<uvec>(Parameter);
	m_C          = *mData;                     
	mat W        = m_C.rows(m_Landmarks);
	W            = 0.5*(W+W.t());
	m_Wpinv      = pinv(W);
	return bStatus;
}

mat NystroemKernelMatrix::Evaluate(const uvec &X, const uvec &Y) const
{
	return m_C.rows(X) * m_Wpinv * (m_C.rows(Y)).t();
}


void NystroemKernelMatrix::size(unsigned int &rows, unsigned int &cols) const
{
	rows = m_C.n_rows;
	cols = rows;
}

/*mat NystroemKernelMatrix::GetParameter (int iParameter) const
{
	switch(iParameter)
	{
		case 1: 
				return m_C.rows(m_vXSelector);
		case 2: 
				return m_Wpinv;
		case 3: 
				return conv_to<mat>::from(m_Landmarks);
		case 4:
				return conv_to<mat>::from(m_vXSelector);
		case 5:
				return conv_to<mat>::from(m_vYSelector);		
	}
}*/
