/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*-  */
/*
 * omp.cpp
 * Copyright (C) 2015 Frank-Michael Schleif <schleify@cs.bham.ac.uk>
 *
 * This is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * The code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ompursuit.h"
#include <iostream>
#include <limits>
#include <armadillo>

#include <iostream>
#include <boost/bind.hpp>
#include <assert.h>
#include <fstream>
#include <boost/shared_ptr.hpp>
#include "nystroem-kernel-matrix.h"

using namespace std;
using namespace arma;


vec omp::OmpSolver(const IKernelMatrixInterface &oKernelInstance, const colvec &y, unsigned int &k)
{
	unsigned int rows=0, cols = 0;
	oKernelInstance.size(rows, cols);

	vec x          = zeros<vec>(rows,1);
    uvec support;
    colvec temp     = y;
    int count       = 1;
	int iKernelType = oKernelInstance.GetKernelType();
	vec xfinal;
	if(iKernelType == 3) // Nystroem
	{
		while(count < k+1)
		{
			const mat& C = (dynamic_cast<const NystroemKernelMatrix&>(oKernelInstance)).GetParameterC();
			const mat& W = (dynamic_cast<const NystroemKernelMatrix&>(oKernelInstance)).GetParameterP();
			colvec ST    = abs(C*((C.t()*temp).t()*W).t());
			uword	   b = ST.index_max();
			support      = join_cols(support,uvec({b}));
			mat subA     = C*W*C.rows(support).t();
			xfinal       = pinv(subA) * y;
			temp         = (y-subA * xfinal);
			count++;	
		}
		x(support) = xfinal;
	}
	else
	{
		uvec vTemp = linspace<ucolvec>(0,rows-1,rows);
		mat A      = oKernelInstance.Evaluate(vTemp,vTemp);
		while(count < k+1)
		{
			colvec ST    = abs(A*temp);	
			uword	   b = ST.index_max();			
			support      = join_cols(support,uvec({b}));
			mat subA     = A.cols(support);
			xfinal       = pinv(subA) * y;
			temp         = (y-subA * xfinal);
			count++;	
		}	
		x(support) = xfinal;

	}
	return x;
}

