/**
 * @file iqp_test.cpp
 * @author Frank-Michael Schleif
 *
 * Test the iqp code 
 *
 */
#include <boost/bind.hpp>
#include <boost/test/unit_test.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>
#include "old_boost_test_definitions.hpp"
#include "iqp_solver.h"
#define BOOST_TEST_DYN_LINK


BOOST_AUTO_TEST_SUITE(IQPMethodTest);


/**
 * description goes here
 */
BOOST_AUTO_TEST_CASE(IQPTest1)
{
  iqpsolver iqpSolverInstance;
  vec f = { -2, -6 };
  vec b = { 2, 2, 3 };

  mat H = { {1, -1},
            {-1, 2} };

  mat A = { {1, 1},
            {-1, 2},
	    {2, 1} };

  double dVal = 0;
  colvec vAlpha;	

  int iResult = iqpSolverInstance.InteriorPointSolver(H, f, A, b, true, vAlpha, dVal);
  std::cout << dVal << std::endl;
  std::cout << vAlpha << std::endl;
  std::cout << abs(abs(dVal)-8.222) << std::endl;
  BOOST_CHECK( abs(abs(dVal)-8.222)<1e-2 );


  f = { 3, 5 };
  b = { 100 };

  H = { {2, 1},
            {1, 2} };

  A = { {2, 3} };

  dVal = 0;  

  iResult = iqpSolverInstance.InteriorPointSolver(H, f, A, b, true, vAlpha, dVal);
  std::cout << dVal << std::endl;
  std::cout << vAlpha << std::endl;
  std::cout << abs(abs(dVal)-6.333) << std::endl;
  BOOST_CHECK( abs(abs(dVal)-6.333)<1e-2 );


  H = { {6.0132,    2.3061,  -3.4679},
        {2.3061,    7.5275,   -3.1938},
        {-3.4679,   -3.1938,   10.3383}};

  f = {0, 0, 0};

  A = { { 1.0000, 0,      0},
        { 0     , 1.0000, 0},
        { 0     , 0,      1.0000},
        {1.0000 , 1.0000, 1.0000},
        {1.0000 , 1.0000, 1.0000} };

  b = {0, 0, 0, 0, -1.0000};
  dVal = 0;  

  iResult = iqpSolverInstance.InteriorPointSolver(H, f, A, b, true, vAlpha, dVal);
  std::cout << dVal << std::endl;
  std::cout << vAlpha << std::endl;
  std::cout << abs(abs(dVal)-0.812709024827177) << std::endl;
  BOOST_CHECK( abs(abs(dVal)-0.812709024827177)<1e-2 );

}

BOOST_AUTO_TEST_SUITE_END();
